@@ -1,5 +1,5 @@
 """
-Copyright  2016-2020 Maël Azimi <m.a@moul.re>
+Copyright  2016-2021 Maël Azimi <m.a@moul.re>
 
 Silkaj is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as published by
@@ -16,7 +16,7 @@ along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 """
 
 import sys
-from click import command, argument, echo, confirm
+import click
 from time import time
 from tabulate import tabulate
 from duniterpy.api import bma
@@ -24,19 +24,19 @@ from duniterpy.documents import BlockUID, block_uid, Identity, Certification
 
 from silkaj.auth import auth_method
 from silkaj.tools import message_exit, coroutine
-from silkaj.tui import convert_time, display_pubkey_and_checksum
 from silkaj.network_tools import ClientInstance
 from silkaj.blockchain_tools import BlockchainParams, HeadBlock
 from silkaj.license import license_approval
-from silkaj import wot
+from silkaj import wot, tui
 from silkaj.constants import SUCCESS_EXIT_STATUS
 from silkaj.crypto_tools import is_pubkey_and_check
 
 
-@command("cert", help="Send certification")
-@argument("uid_pubkey_to_certify")
+@click.command("cert", help="Send certification")
+@click.argument("uid_pubkey_to_certify")
+@click.pass_context
 @coroutine
-async def send_certification(uid_pubkey_to_certify):
+async def send_certification(ctx, uid_pubkey_to_certify):
     client = ClientInstance().client
 
     checked_pubkey = is_pubkey_and_check(uid_pubkey_to_certify)
@@ -50,8 +50,47 @@ async def send_certification(uid_pubkey_to_certify):
     # Authentication
     key = auth_method()
 
-    # Check whether current user is member
     issuer_pubkey = key.pubkey
+    issuer = await pre_checks(client, issuer_pubkey, pubkey_to_certify)
+
+    # Display license and ask for confirmation
+    head = await HeadBlock().head_block
+    currency = head["currency"]
+    license_approval(currency)
+
+    # Certification confirmation
+    await certification_confirmation(
+        ctx, issuer, issuer_pubkey, pubkey_to_certify, idty_to_certify
+    )
+
+    certification = docs_generation(
+        currency,
+        pubkey_to_certify,
+        idty_to_certify,
+        issuer_pubkey,
+        head,
+    )
+
+    if ctx.obj["DISPLAY_DOCUMENT"]:
+        click.echo(certification.signed_raw(), nl=False)
+        await tui.send_doc_confirmation("certification")
+
+    # Sign document
+    certification.sign([key])
+
+    # Send certification document
+    response = await client(bma.wot.certify, certification.signed_raw())
+
+    if response.status == 200:
+        print("Certification successfully sent.")
+    else:
+        print("Error while publishing certification: {0}".format(await response.text()))
+
+    await client.close()
+
+
+async def pre_checks(client, issuer_pubkey, pubkey_to_certify):
+    # Check whether current user is member
     issuer = await wot.is_member(issuer_pubkey)
     if not issuer:
         message_exit("Current identity is not member.")
@@ -69,24 +108,47 @@ async def send_certification(uid_pubkey_to_certify):
             # ĞT: 0<–>4.8m - 4.8m + 12.5d
             renewable = cert["expiresIn"] - params["sigValidity"] + params["sigReplay"]
             if renewable > 0:
-                renewable_date = convert_time(time() + renewable, "date")
+                renewable_date = tui.convert_time(time() + renewable, "date")
                 message_exit("Certification renewable the " + renewable_date)
 
     # Check if the certification is already in the pending certifications
     for pending_cert in req["pendingCerts"]:
         if pending_cert["from"] == issuer_pubkey:
             message_exit("Certification is currently been processed")
+    return issuer
 
-    # Display license and ask for confirmation
-    head = await HeadBlock().head_block
-    currency = head["currency"]
-    license_approval(currency)
 
-    # Certification confirmation
-    await certification_confirmation(
-        issuer, issuer_pubkey, pubkey_to_certify, idty_to_certify
+async def certification_confirmation(
+    ctx, issuer, issuer_pubkey, pubkey_to_certify, idty_to_certify
+):
+    cert = list()
+    cert.append(["Cert", "Issuer", "–>", "Recipient: Published: #block-hash date"])
+    client = ClientInstance().client
+    idty_timestamp = idty_to_certify["meta"]["timestamp"]
+    block_uid_idty = block_uid(idty_timestamp)
+    block = await client(bma.blockchain.block, block_uid_idty.number)
+    block_uid_date = (
+        ": #" + idty_timestamp[:15] + "… " + tui.convert_time(block["time"], "all")
     )
+    cert.append(["ID", issuer["uid"], "–>", idty_to_certify["uid"] + block_uid_date])
+    cert.append(
+        [
+            "Pubkey",
+            tui.display_pubkey_and_checksum(issuer_pubkey),
+            "–>",
+            tui.display_pubkey_and_checksum(pubkey_to_certify),
+        ]
+    )
+    params = await BlockchainParams().params
+    cert_begins = tui.convert_time(time(), "date")
+    cert_ends = tui.convert_time(time() + params["sigValidity"], "date")
+    cert.append(["Valid", cert_begins, "—>", cert_ends])
+    click.echo(tabulate(cert, tablefmt="fancy_grid"))
+    if not ctx.obj["DISPLAY_DOCUMENT"]:
+        await tui.send_doc_confirmation("certification")
 
+
+def docs_generation(currency, pubkey_to_certify, idty_to_certify, issuer_pubkey, head):
     identity = Identity(
         version=10,
         currency=currency,
@@ -96,7 +158,7 @@ async def send_certification(uid_pubkey_to_certify):
         signature=idty_to_certify["self"],
     )
 
-    certification = Certification(
+    return Certification(
         version=10,
         currency=currency,
         pubkey_from=issuer_pubkey,
@@ -104,47 +166,3 @@ async def send_certification(uid_pubkey_to_certify):
         timestamp=BlockUID(head["number"], head["hash"]),
         signature="",
     )
-
-    # Sign document
-    certification.sign([key])
-
-    # Send certification document
-    response = await client(bma.wot.certify, certification.signed_raw())
-
-    if response.status == 200:
-        print("Certification successfully sent.")
-    else:
-        print("Error while publishing certification: {0}".format(await response.text()))
-
-    await client.close()
-
-
-async def certification_confirmation(
-    issuer, issuer_pubkey, pubkey_to_certify, idty_to_certify
-):
-    cert = list()
-    cert.append(["Cert", "Issuer", "–>", "Recipient: Published: #block-hash date"])
-    client = ClientInstance().client
-    idty_timestamp = idty_to_certify["meta"]["timestamp"]
-    block_uid_idty = block_uid(idty_timestamp)
-    block = await client(bma.blockchain.block, block_uid_idty.number)
-    block_uid_date = (
-        ": #" + idty_timestamp[:15] + "… " + convert_time(block["time"], "all")
-    )
-    cert.append(["ID", issuer["uid"], "–>", idty_to_certify["uid"] + block_uid_date])
-    cert.append(
-        [
-            "Pubkey",
-            display_pubkey_and_checksum(issuer_pubkey),
-            "–>",
-            display_pubkey_and_checksum(pubkey_to_certify),
-        ]
-    )
-    params = await BlockchainParams().params
-    cert_begins = convert_time(time(), "date")
-    cert_ends = convert_time(time() + params["sigValidity"], "date")
-    cert.append(["Valid", cert_begins, "—>", cert_ends])
-    echo(tabulate(cert, tablefmt="fancy_grid"))
-    if not confirm("Do you confirm sending this certification?"):
-        await client.close()
-        sys.exit(SUCCESS_EXIT_STATUS)
