@@ -1,5 +1,5 @@
 """
-Copyright  2016-2020 Maël Azimi <m.a@moul.re>
+Copyright  2016-2021 Maël Azimi <m.a@moul.re>
 
 Silkaj is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as published by
@@ -101,7 +101,7 @@ async def difficulties():
             current = await ws.receive_json()
             jsonschema.validate(current, bma.ws.WS_BLOCK_SCHEMA)
             diffi = await client(bma.blockchain.difficulties)
-            await display_diffi(current, diffi)
+            display_diffi(current, diffi)
         await client.close()
 
     except (aiohttp.WSServerHandshakeError, ValueError) as e:
@@ -112,7 +112,7 @@ async def difficulties():
         print("{:}:{:}".format(str(e.__class__.__name__), str(e)))
 
 
-async def display_diffi(current, diffi):
+def display_diffi(current, diffi):
     levels = [OrderedDict((i, d[i]) for i in ("uid", "level")) for d in diffi["levels"]]
     diffi["levels"] = levels
     issuers = 0
@@ -233,7 +233,8 @@ async def list_blocks(number, detailed):
 @coroutine
 async def argos_info():
     head_block = await HeadBlock().head_block
-    print(await CurrencySymbol().symbol, "|")
+    currency_symbol = await CurrencySymbol().symbol
+    print(currency_symbol, "|")
     print("---")
     ep = EndPoint().ep
     endpoint_address = ep[best_endpoint_address(ep, False)]
@@ -250,7 +251,7 @@ async def argos_info():
         "\nCurrent block number:",
         head_block["number"],
         "\nCurrency name:",
-        await CurrencySymbol().symbol,
+        currency_symbol,
         "\nNumber of members:",
         head_block["membersCount"],
         "\nMinimal Proof-of-Work:",
