@@ -1,5 +1,5 @@
 """
-Copyright  2016-2020 Maël Azimi <m.a@moul.re>
+Copyright  2016-2021 Maël Azimi <m.a@moul.re>
 
 Silkaj is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as published by
@@ -16,13 +16,16 @@ along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 """
 
 import re
-from nacl import encoding, hash
+import hashlib
+
+import base58
 
 from silkaj.constants import PUBKEY_PATTERN
 from silkaj.tools import message_exit
 
 PUBKEY_DELIMITED_PATTERN = "^{0}$".format(PUBKEY_PATTERN)
-CHECKSUM_PATTERN = "[1-9A-HJ-NP-Za-km-z]{3}"
+CHECKSUM_SIZE = 3
+CHECKSUM_PATTERN = f"[1-9A-HJ-NP-Za-km-z]{{{CHECKSUM_SIZE}}}"
 PUBKEY_CHECKSUM_PATTERN = "^{0}:{1}$".format(PUBKEY_PATTERN, CHECKSUM_PATTERN)
 
 
@@ -76,64 +79,6 @@ def gen_checksum(pubkey):
     """
     Returns the checksum of the input pubkey (encoded in b58)
     """
-    pubkey_byte = b58_decode(pubkey)
-    return b58_encode(
-        hash.sha256(hash.sha256(pubkey_byte, encoding.RawEncoder), encoding.RawEncoder)
-    )[:3]
-
-
-b58_digits = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
-
-
-def b58_encode(b):
-
-    # Convert big-endian bytes to integer
-    n = int("0x0" + encoding.HexEncoder.encode(b).decode("utf8"), 16)
-
-    # Divide that integer into base58
-    res = []
-    while n > 0:
-        n, r = divmod(n, 58)
-        res.append(b58_digits[r])
-    res = "".join(res[::-1])
-
-    # Encode leading zeros as base58 zeros
-    czero = 0
-    pad = 0
-    for c in b:
-        if c == czero:
-            pad += 1
-        else:
-            break
-    return b58_digits[0] * pad + res
-
-
-def b58_decode(s):
-    if not s:
-        return b""
-
-    # Convert the string to an integer
-    n = 0
-    for c in s:
-        n *= 58
-        if c not in b58_digits:
-            raise InvalidBase58Error(
-                "Character %r is not a " + "valid base58 character" % c
-            )
-        digit = b58_digits.index(c)
-        n += digit
-
-    # Convert the integer to bytes
-    h = "%x" % n
-    if len(h) % 2:
-        h = "0" + h
-    res = encoding.HexEncoder.decode(h.encode("utf8"))
-
-    # Add padding back.
-    pad = 0
-    for c in s[:-1]:
-        if c == b58_digits[0]:
-            pad += 1
-        else:
-            break
-    return b"\x00" * pad + res
+    pubkey_byte = base58.b58decode(pubkey)
+    hash = hashlib.sha256(hashlib.sha256(pubkey_byte).digest()).digest()
+    return base58.b58encode(hash)[:3].decode("utf-8")
