@@ -1,5 +1,5 @@
 """
-Copyright  2016-2020 Maël Azimi <m.a@moul.re>
+Copyright  2016-2021 Maël Azimi <m.a@moul.re>
 
 Silkaj is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as published by
@@ -19,59 +19,74 @@ from re import compile, search
 import math
 from asyncio import sleep
 from tabulate import tabulate
-from click import command, option, FloatRange
+import click
 
-from silkaj.cli_tools import MutuallyExclusiveOption
-from silkaj.network_tools import ClientInstance
-from silkaj.blockchain_tools import HeadBlock
-from silkaj.crypto_tools import validate_checksum, check_pubkey_format
-from silkaj.tools import message_exit, CurrencySymbol, coroutine
-from silkaj.auth import auth_method
+from silkaj import cli_tools
+from silkaj import network_tools as nt
+from silkaj import crypto_tools as ct
+from silkaj import blockchain_tools as bt
+from silkaj import tools
+from silkaj import auth
 from silkaj import money
+from silkaj import tui
 from silkaj.constants import (
-    SOURCES_PER_TX,
-    MINIMAL_TX_AMOUNT,
+    MINIMAL_ABSOLUTE_TX_AMOUNT,
+    MINIMAL_RELATIVE_TX_AMOUNT,
     CENT_MULT_TO_UNIT,
     ASYNC_SLEEP,
 )
-from silkaj.tui import display_amount, display_pubkey, display_pubkey_and_checksum
 
 from duniterpy.api.bma.tx import process
 from duniterpy.documents import BlockUID, Transaction
 from duniterpy.documents.transaction import OutputSource, Unlock, SIGParameter
 
+MAX_COMMENT_LENGTH = 255
 
-@command("tx", help="Send transaction")
-@option(
+
+# max size for tx doc is 100 lines. Formula for accepted field numbers is : (2 * IU + 2 * IS + OUT) <= ( MAX_LINES_IN_TX_DOC - FIX_LINES)
+# with IU = inputs/unlocks ; IS = Issuers/Signatures ; OUT = Outpouts.
+MAX_LINES_IN_TX_DOC = 100
+# 2 lines are necessary, and we block 1 more for the comment
+FIX_LINES = 3
+# assuming there is only 1 issuer and 2 outputs, max inputs is 46
+MAX_INPUTS_PER_TX = 46
+# assuming there is 1 issuer and 1 input, max outputs is 93.
+MAX_OUTPUTS = 93
+# for now, silkaj handles txs for one issuer only
+NBR_ISSUERS = 1
+
+
+@click.command("tx", help="Send transaction")
+@click.option(
     "amounts",
     "--amount",
     "-a",
     multiple=True,
-    type=FloatRange(MINIMAL_TX_AMOUNT),
+    type=click.FloatRange(MINIMAL_ABSOLUTE_TX_AMOUNT),
     help="Quantitative amount(s):\n-a <amount>\nMinimum amount is {0}".format(
-        MINIMAL_TX_AMOUNT
+        MINIMAL_ABSOLUTE_TX_AMOUNT
     ),
-    cls=MutuallyExclusiveOption,
+    cls=cli_tools.MutuallyExclusiveOption,
     mutually_exclusive=["amountsud", "allsources"],
 )
-@option(
+@click.option(
     "amountsud",
     "--amountUD",
     "-d",
     multiple=True,
-    type=float,
-    help="Relative amount(s):\n-d <amount_UD>",
-    cls=MutuallyExclusiveOption,
+    type=click.FloatRange(MINIMAL_RELATIVE_TX_AMOUNT),
+    help=f"Relative amount(s):\n-d <amount_UD>\nMinimum amount is {MINIMAL_RELATIVE_TX_AMOUNT}",
+    cls=cli_tools.MutuallyExclusiveOption,
     mutually_exclusive=["amounts", "allsources"],
 )
-@option(
+@click.option(
     "--allSources",
     is_flag=True,
     help="Send all sources to one recipient",
-    cls=MutuallyExclusiveOption,
+    cls=cli_tools.MutuallyExclusiveOption,
     mutually_exclusive=["amounts", "amountsud"],
 )
-@option(
+@click.option(
     "recipients",
     "--recipient",
     "-r",
@@ -82,34 +97,47 @@ Sending to many recipients is possible:\n\
 * With one amount, all will receive the amount\n\
 * With many amounts (one per recipient)",
 )
-@option("--comment", "-c", default="", help="Comment")
-@option(
+@click.option("--comment", "-c", default="", help="Comment")
+@click.option(
     "--outputBackChange",
     help="Pubkey recipient to send the rest of the transaction: <pubkey[:checksum]>",
 )
-@option("--yes", "-y", is_flag=True, help="Assume yes. Do not prompt confirmation")
-@coroutine
+@click.option(
+    "--yes", "-y", is_flag=True, help="Assume yes. Do not prompt confirmation"
+)
+@tools.coroutine
 async def send_transaction(
-    amounts, amountsud, allsources, recipients, comment, outputbackchange, yes
+    amounts,
+    amountsud,
+    allsources,
+    recipients,
+    comment,
+    outputbackchange,
+    yes,
 ):
     """
     Main function
     """
     if not (amounts or amountsud or allsources):
-        message_exit("Error: amount, amountUD or allSources is not set.")
+        tools.message_exit("Error: amount, amountUD or allSources is not set.")
     if allsources and len(recipients) > 1:
-        message_exit(
+        tools.message_exit(
             "Error: the --allSources option can only be used with one recipient."
         )
     # compute amounts and amountsud
     if not allsources:
         tx_amounts = await transaction_amount(amounts, amountsud, recipients)
 
-    key = auth_method()
+    key = auth.auth_method()
     issuer_pubkey = key.pubkey
 
     pubkey_amount = await money.get_amount_from_pubkey(issuer_pubkey)
     if allsources:
+        if pubkey_amount[0] <= 0:
+            tools.message_exit(
+                f"Error: Issuer pubkey {tui.display_pubkey_and_checksum(issuer_pubkey)} is empty. No transaction sent."
+            )
+
         tx_amounts = [pubkey_amount[0]]
 
     recipients = list(recipients)
@@ -121,23 +149,21 @@ async def send_transaction(
         issuer_pubkey,
     )
 
-    if (
-        yes
-        or input(
-            tabulate(
-                await transaction_confirmation(
-                    issuer_pubkey,
-                    pubkey_amount[0],
-                    tx_amounts,
-                    recipients,
-                    outputbackchange,
-                    comment,
-                ),
-                tablefmt="fancy_grid",
-            )
-            + "\nDo you confirm sending this transaction? [yes/no]: "
+    if not yes:
+        confirmation_table = tabulate(
+            await gen_confirmation_table(
+                issuer_pubkey,
+                pubkey_amount[0],
+                tx_amounts,
+                recipients,
+                outputbackchange,
+                comment,
+            ),
+            tablefmt="fancy_grid",
         )
-        == "yes"
+
+    if yes or click.confirm(
+        f"{confirmation_table}\nDo you confirm sending this transaction?"
     ):
         await handle_intermediaries_transactions(
             key,
@@ -148,7 +174,7 @@ async def send_transaction(
             outputbackchange,
         )
     else:
-        client = ClientInstance().client
+        client = nt.ClientInstance().client
         await client.close()
 
 
@@ -164,7 +190,7 @@ async def transaction_amount(amounts, UDs_amounts, outputAddresses):
         UD_value = await money.UDValue().ud_value
         amounts_list = compute_amounts(UDs_amounts, UD_value)
     if len(amounts_list) != len(outputAddresses) and len(amounts_list) != 1:
-        message_exit(
+        tools.message_exit(
             "Error: The number of passed recipients is not the same as the passed amounts."
         )
     # In case one amount is passed with multiple recipients
@@ -186,9 +212,9 @@ def compute_amounts(amounts, multiplicator):
         computed_amount = amount * multiplicator
         # check if relative amounts are high enough
         if (multiplicator != CENT_MULT_TO_UNIT) and (
-            computed_amount < (MINIMAL_TX_AMOUNT * CENT_MULT_TO_UNIT)
+            computed_amount < (MINIMAL_ABSOLUTE_TX_AMOUNT * CENT_MULT_TO_UNIT)
         ):
-            message_exit("Error: amount {0} is too low.".format(amount))
+            tools.message_exit("Error: amount {0} is too low.".format(amount))
         amounts_list.append(round(computed_amount))
     return amounts_list
 
@@ -203,21 +229,26 @@ def check_transaction_values(
     Check the balance is big enough for the transaction
     """
     checkComment(comment)
+    # we check output numbers and leave one line for the backchange.
+    if len(outputAddresses) > (MAX_OUTPUTS - 1):
+        tools.message_exit(
+            "Error : there should be less than {0} outputs.".format(MAX_OUTPUTS - 1)
+        )
     for i, outputAddress in enumerate(outputAddresses):
-        if check_pubkey_format(outputAddress):
-            outputAddresses[i] = validate_checksum(outputAddress)
+        if ct.check_pubkey_format(outputAddress):
+            outputAddresses[i] = ct.validate_checksum(outputAddress)
     if outputBackChange:
-        if check_pubkey_format(outputBackChange):
-            outputBackChange = validate_checksum(outputBackChange)
+        if ct.check_pubkey_format(outputBackChange):
+            outputBackChange = ct.validate_checksum(outputBackChange)
     if enough_source:
-        message_exit(
-            display_pubkey_and_checksum(issuer_pubkey)
+        tools.message_exit(
+            tui.display_pubkey_and_checksum(issuer_pubkey)
             + " pubkey doesn’t have enough money for this transaction."
         )
     return outputBackChange
 
 
-async def transaction_confirmation(
+async def gen_confirmation_table(
     issuer_pubkey,
     pubkey_amount,
     tx_amounts,
@@ -229,64 +260,74 @@ async def transaction_confirmation(
     Generate transaction confirmation
     """
 
-    currency_symbol = await CurrencySymbol().symbol
+    currency_symbol = await tools.CurrencySymbol().symbol
     ud_value = await money.UDValue().ud_value
     total_tx_amount = sum(tx_amounts)
     tx = list()
     # display account situation
-    display_amount(
+    tui.display_amount(
         tx,
         "Initial balance",
         pubkey_amount,
         ud_value,
         currency_symbol,
     )
-    display_amount(
+    tui.display_amount(
         tx,
         "Total transaction amount",
         total_tx_amount,
         ud_value,
         currency_symbol,
     )
-    display_amount(
+    tui.display_amount(
         tx,
         "Balance after transaction",
         (pubkey_amount - total_tx_amount),
         ud_value,
         currency_symbol,
     )
-    await display_pubkey(tx, "From", issuer_pubkey)
+    await tui.display_pubkey(tx, "From", issuer_pubkey)
     # display outputs and amounts
     for outputAddress, tx_amount in zip(outputAddresses, tx_amounts):
-        await display_pubkey(tx, "To", outputAddress)
+        await tui.display_pubkey(tx, "To", outputAddress)
         await sleep(ASYNC_SLEEP)
-        display_amount(tx, "Amount", tx_amount, ud_value, currency_symbol)
+        tui.display_amount(tx, "Amount", tx_amount, ud_value, currency_symbol)
     # display last informations
     if outputBackChange:
-        await display_pubkey(tx, "Backchange", outputBackChange)
+        await tui.display_pubkey(tx, "Backchange", outputBackChange)
     tx.append(["Comment", comment])
     return tx
 
 
-async def get_list_input_for_transaction(pubkey, TXamount):
+async def get_list_input_for_transaction(pubkey, TXamount, outputs_number):
     listinput, amount = await money.get_sources(pubkey)
-
+    maxInputsNumber = max_inputs_number(outputs_number, NBR_ISSUERS)
     # generate final list source
     listinputfinal = []
     totalAmountInput = 0
     intermediatetransaction = False
-    for input in listinput:
+    for nbr_inputs, input in enumerate(listinput, start=1):
         listinputfinal.append(input)
         totalAmountInput += money.amount_in_current_base(input)
         TXamount -= money.amount_in_current_base(input)
-        # if more than 40 sources, it's an intermediate transaction
-        if len(listinputfinal) >= SOURCES_PER_TX:
+        # if too much sources, it's an intermediate transaction.
+        amount_not_reached_and_max_doc_size_reached = (
+            TXamount > 0 and MAX_INPUTS_PER_TX <= nbr_inputs
+        )
+        amount_reached_too_much_inputs = TXamount <= 0 and maxInputsNumber < nbr_inputs
+        if (
+            amount_not_reached_and_max_doc_size_reached
+            or amount_reached_too_much_inputs
+        ):
             intermediatetransaction = True
-            break
-        if TXamount <= 0:
+        # if we reach the MAX_INPUTX_PER_TX limit, we send the interm.tx
+        # if we gather the good amount, we send the tx :
+        #    - either this is no int.tx, and the tx is sent to the receiver,
+        #    - or the int.tx it is sent to the issuer before sent to the receiver.
+        if MAX_INPUTS_PER_TX <= nbr_inputs or TXamount <= 0:
             break
     if TXamount > 0 and not intermediatetransaction:
-        message_exit("Error: you don't have enough money")
+        tools.message_exit("Error: you don't have enough money")
     return listinputfinal, totalAmountInput, intermediatetransaction
 
 
@@ -298,10 +339,12 @@ async def handle_intermediaries_transactions(
     Comment="",
     OutputbackChange=None,
 ):
-    client = ClientInstance().client
+    client = nt.ClientInstance().client
+
     while True:
+        # consider there is always one backchange output, hence +1
         listinput_and_amount = await get_list_input_for_transaction(
-            issuers, sum(tx_amounts)
+            issuers, sum(tx_amounts), len(outputAddresses) + 1
         )
         intermediatetransaction = listinput_and_amount[2]
 
@@ -331,6 +374,17 @@ async def handle_intermediaries_transactions(
             break
 
 
+def max_inputs_number(outputs_number, issuers_number):
+    """
+    returns the maximum number of inputs.
+    This function does not take care of backchange line.
+    formula is IU <= (MAX_LINES_IN_TX_DOC - FIX_LINES - O - 2*IS)/2
+    """
+    return int(
+        (MAX_LINES_IN_TX_DOC - FIX_LINES - (2 * issuers_number) - outputs_number) / 2
+    )
+
+
 async def generate_and_send_transaction(
     key,
     issuers,
@@ -349,12 +403,12 @@ async def generate_and_send_transaction(
         print("Generate Change Transaction")
     else:
         print("Generate Transaction:")
-    print("   - From:    " + display_pubkey_and_checksum(issuers))
+    print("   - From:    " + tui.display_pubkey_and_checksum(issuers))
     for tx_amount, outputAddress in zip(tx_amounts, outputAddresses):
         display_sent_tx(outputAddress, tx_amount)
     print("   - Total:   " + str(sum(tx_amounts) / 100))
 
-    client = ClientInstance().client
+    client = nt.ClientInstance().client
     transaction = await generate_transaction_document(
         issuers,
         tx_amounts,
@@ -364,11 +418,12 @@ async def generate_and_send_transaction(
         OutputbackChange,
     )
     transaction.sign([key])
+
     response = await client(process, transaction.signed_raw())
     if response.status == 200:
         print("Transaction successfully sent.")
     else:
-        message_exit(
+        tools.message_exit(
             "Error while publishing transaction: {0}".format(await response.text())
         )
 
@@ -376,7 +431,7 @@ async def generate_and_send_transaction(
 def display_sent_tx(outputAddress, amount):
     print(
         "   - To:     ",
-        display_pubkey_and_checksum(outputAddress),
+        tui.display_pubkey_and_checksum(outputAddress),
         "\n   - Amount: ",
         amount / 100,
     )
@@ -395,7 +450,7 @@ async def generate_transaction_document(
     totalAmountInput = listinput_and_amount[1]
     total_tx_amount = sum(tx_amounts)
 
-    head_block = await HeadBlock().head_block
+    head_block = await bt.HeadBlock().head_block
     currency_name = head_block["currency"]
     blockstamp_current = BlockUID(head_block["number"], head_block["hash"])
     curentUnitBase = head_block["unitbase"]
@@ -462,14 +517,14 @@ def generate_output(listoutput, unitbase, rest, recipient_address):
         unitbase = unitbase - 1
 
 
-def checkComment(Comment):
-    if len(Comment) > 255:
-        message_exit("Error: Comment is too long")
+def checkComment(comment):
+    if len(comment) > MAX_COMMENT_LENGTH:
+        tools.message_exit("Error: Comment is too long")
     regex = compile(
         "^[0-9a-zA-Z\\ \\-\\_\\:\\/\\;\\*\\[\\]\\(\\)\\?\\!\\^\\+\\=\\@\\&\\~\\#\\{\\}\\|\\\\<\\>\\%\\.]*$"
     )
-    if not search(regex, Comment):
-        message_exit("Error: the format of the comment is invalid")
+    if not search(regex, comment):
+        tools.message_exit("Error: the format of the comment is invalid")
 
 
 def truncBase(amount, base):
